@@ -1,6 +1,7 @@
 from __future__ import annotations
 
 import datetime as dt
+import json
 from contextlib import contextmanager
 from dataclasses import asdict, is_dataclass
 from pathlib import Path
@@ -34,6 +35,8 @@ class DbtBaseOperator(BaseOperator):
     variables defined in dbt_profile.yml.
         log_cache_events: Flag to enable logging of cache events.
         bypass_cache: Flag to bypass the adapter-level cache of database state.
+        s3_conn_id: An s3 Airflow connection ID to use when pulling dbt files from s3.
+        do_xcom_push_artifacts: A list of dbt artifacts to XCom push.
 
         Methods:
         execute: Executes a given dbt command.
@@ -42,6 +45,12 @@ class DbtBaseOperator(BaseOperator):
         serializable_result: Turns a dbt result into a serializable object.
     """
 
+    template_fields = [
+        "project_dir",
+        "profiles_dir",
+        "profile",
+        "target",
+    ]
     command: Optional[str] = None
     __dbt_args__ = [
         "project_dir",
@@ -65,6 +74,7 @@ class DbtBaseOperator(BaseOperator):
         log_cache_events: Optional[bool] = False,
         bypass_cache: Optional[bool] = False,
         s3_conn_id: str = "aws_default",
+        do_xcom_push_artifacts: Optional[list[str]] = None,
         **kwargs,
     ) -> None:
         super().__init__(**kwargs)
@@ -77,6 +87,7 @@ class DbtBaseOperator(BaseOperator):
         self.log_cache_events = log_cache_events
         self.bypass_cache = bypass_cache
         self.s3_conn_id = s3_conn_id
+        self.do_xcom_push_artifacts = do_xcom_push_artifacts
         self._dbt_s3_hook = None
 
     def execute(self, context: dict):
@@ -86,14 +97,20 @@ class DbtBaseOperator(BaseOperator):
                 args = self.prepare_args()
                 self.log.info("Running dbt %s with args %s", args[0], args[1:])
 
-                res, success = self.run_dbt_command(args)
+                try:
+                    res, success = self.run_dbt_command(args)
+                except Exception as e:
+                    self.log.exception("There was an error executing dbt", exc_info=e)
+                    res, success = {}, False
 
-        if self.do_xcom_push is True:
-            # Some dbt operations use dataclasses for its results,
-            # found in dbt.contracts.results. Each DbtBaseOperator
-            # subclass should implement prepare_results to return a
-            # serializable object
-            res = self.serializable_result(res)
+                if self.do_xcom_push is True:
+                    # Some dbt operations use dataclasses for its results,
+                    # found in dbt.contracts.results. Each DbtBaseOperator
+                    # subclass should implement prepare_results to return a
+                    # serializable object
+                    res = self.serializable_result(res)
+                    if context.get("ti", None) is not None:
+                        self.xcom_push_artifacts(context, dbt_dir)
 
         if success is not True:
             if self.do_xcom_push is True and context.get("ti", None) is not None:
@@ -101,6 +118,28 @@ class DbtBaseOperator(BaseOperator):
             raise AirflowException(f"dbt {args[0]} {args[1:]} failed")
         return res
 
+    def xcom_push_artifacts(self, context: dict, dbt_directory: str):
+        """
+        Read dbt artifacts from dbt_directory/target/ and push them to XCom
+
+        Arguments:
+        context: The Airflow task's context.
+        dbt_directory: A directory containing a dbt project. Artifacts will be
+            assumed to be in dbt_directory/target/.
+        """
+        if self.do_xcom_push_artifacts is None:
+            # Nothing to xcom_push. Need this for mypy.
+            return
+
+        target_dir = Path(dbt_directory) / "target"
+
+        for artifact in self.do_xcom_push_artifacts:
+            artifact_path = target_dir / artifact
+
+            with open(artifact_path) as f:
+                json_dict = json.load(f)
+            self.xcom_push(context, key=artifact, value=json_dict)
+
     def prepare_args(self) -> list[Optional[str]]:
         """Prepare the arguments needed to call dbt"""
         if self.command is None:
@@ -161,27 +200,29 @@ class DbtBaseOperator(BaseOperator):
         store_project_dir = self.project_dir
 
         with TemporaryDirectory(prefix="airflowtmp") as tmp_dir:
-            if urlparse(str(self.profiles_dir)).scheme == "s3":
-                self.log.info("Fetching profiles.yml from S3")
-                profiles_file_path = self.dbt_s3_hook.get_dbt_profiles(
-                    self.profiles_dir,
-                    tmp_dir,
-                )
-                self.profiles_dir = str(profiles_file_path.parent) + "/"
-
-            if urlparse(str(self.project_dir)).scheme == "s3":
-                self.log.info("Fetching dbt project from S3")
-                project_dir_path = self.dbt_s3_hook.get_dbt_project(
-                    self.project_dir,
-                    tmp_dir,
-                )
-                self.project_dir = str(project_dir_path) + "/"
-
+            self.prepare_directory(tmp_dir)
             yield tmp_dir
 
         self.profiles_dir = store_profiles_dir
         self.project_dir = store_project_dir
 
+    def prepare_directory(self, tmp_dir: str):
+        if urlparse(str(self.profiles_dir)).scheme == "s3":
+            self.log.info("Fetching profiles.yml from S3: %s", self.profiles_dir)
+            profiles_file_path = self.dbt_s3_hook.get_dbt_profiles(
+                self.profiles_dir,
+                tmp_dir,
+            )
+            self.profiles_dir = str(profiles_file_path.parent) + "/"
+
+        if urlparse(str(self.project_dir)).scheme == "s3":
+            self.log.info("Fetching dbt project from S3: %s", self.project_dir)
+            project_dir_path = self.dbt_s3_hook.get_dbt_project(
+                self.project_dir,
+                tmp_dir,
+            )
+            self.project_dir = str(project_dir_path) + "/"
+
     @property
     def dbt_s3_hook(self):
         """Provides an existing DbtS3Hook or creates one"""
